/*******************************************************************************
 * This software is copyright (c) 2018 by
 * 
 *  - TextGrid Consortium (http://www.textgrid.de)
 *  - DAASI International GmbH (http://www.daasi.de)
 *
 * This is free software. You can redistribute it and/or modify it under the
 * terms described in the GNU Lesser General Public License v3 of which you
 * should have received a copy. Otherwise you can download it from
 *
 *   http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 * @copyright TextGrid Consortium (http://www.textgrid.de)
 * @copyright DAASI International GmbH (http://www.daasi.de)
 * @license http://www.gnu.org/licenses/lgpl-3.0.txt GNU Lesser General Public
 * License v3
 * @author Stefan E. Funk (stefan.e.funk@daasi.de)
 ******************************************************************************/

package info.textgrid.middleware.tgauth.tgauthclient;

import static org.junit.Assert.assertTrue;

import java.io.FileInputStream;
import java.net.URL;
import java.util.List;
import java.util.Properties;

import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Ignore;
import org.junit.Test;

import info.textgrid.namespaces.middleware.tgauth.AddMemberRequest;
import info.textgrid.namespaces.middleware.tgauth.AuthenticationFault;
import info.textgrid.namespaces.middleware.tgauth.CreateProjectRequest;
import info.textgrid.namespaces.middleware.tgauth.CreateProjectResponse;
import info.textgrid.namespaces.middleware.tgauth.DeleteProjectRequest;
import info.textgrid.namespaces.middleware.tgauth.GetAllProjectsRequest;
import info.textgrid.namespaces.middleware.tgauth.GetIDsRequest;
import info.textgrid.namespaces.middleware.tgauth.GetIDsResponse;
import info.textgrid.namespaces.middleware.tgauth.GetObjectsRequest;
import info.textgrid.namespaces.middleware.tgauth.GetRightsRequest;
import info.textgrid.namespaces.middleware.tgauth.GetUserRoleRequest;
import info.textgrid.namespaces.middleware.tgauth.GetUserRoleResponse;
import info.textgrid.namespaces.middleware.tgauth.NotEmptyFault;
import info.textgrid.namespaces.middleware.tgauth.OperationsetResponse;
import info.textgrid.namespaces.middleware.tgauth.PortTgextra;
import info.textgrid.namespaces.middleware.tgauth.ProjectInfo;
import info.textgrid.namespaces.middleware.tgauth.RbacFault;
import info.textgrid.namespaces.middleware.tgauth.ResourcesetResponse;
import info.textgrid.namespaces.middleware.tgauth.RolesetResponse;
import info.textgrid.namespaces.middleware.tgauth.TgAssignedRolesRequest;
import info.textgrid.namespaces.middleware.tgauth.TgCheckAccessRequest;
import info.textgrid.namespaces.middleware.tgauth.UnknownProjectFault;
import info.textgrid.namespaces.middleware.tgauth.UserDetail;
import info.textgrid.namespaces.middleware.tgauth.UserExistsRequest;
import info.textgrid.namespaces.middleware.tgauth.UserRole;

/*******************************************************************************
 * TODOLOG
 * 
 * 	TODO	Add missing methods!
 * 
 *******************************************************************************
 * CHANGELOG
 * 
 *	2016-08-11	Funk	Using TGAuth client from TGAuth service utilities now!
 * 	2015-01-23	Funk	Combined WSDL's again :-)
 * 	2015-01-22	Funk	Separated the two WSDL's (Tgextra and Tgextra-crud).
 *
 ******************************************************************************/

/*******************************************************************************
 * <p>
 * This are TG-crud online tests for testing a running service to fulfil its
 * duties.
 * </p>
 * 
 * @author Stefan E. Funk, DAASI International GmbH
 * @version 2018-02-08
 * @since 2010-09-07
 ******************************************************************************/

public class TestTGAuthServiceOnline {

	// **
	// STATIC FINALS
	// **

	private static final String propertiesFile = "tgauth.test.textgridlaborg.properties";

	// **
	// STATICS
	// **

	private static PortTgextra	tgextra;
	private static String		rbacSessionId;
	private static String		rbacSessionIdInvalid;
	private static String		crudSecret;
	private static String		crudSecretInvalid;
	private static String		logParameter;
	private static String		projectId;
	private static String		projectIdInvalid;
	private static String		username;
	private static String		usernameInvalid;
	private static String		usernameSearch;
	private static String		junitProject;

	/**
	 * @throws java.lang.Exception
	 */
	@BeforeClass
	public static void setUpBeforeClass() throws Exception {

		// Load properties file.
		Properties p = new Properties();
		p.load(new FileInputStream(
				TGAuthClientCommon.getResource(propertiesFile)));

		// Get and create the tgextra stubs.
		URL location = new URL(p.getProperty("tgextraServiceEndpointUrl"));
		URL locationCrud = new URL(
				p.getProperty("tgextraCrudServiceEndpointUrl"));

		System.out.println("WSDL location tgextra: " + location.getProtocol()
				+ "://" + location.getHost() + ":"
				+ (location.getPort() == -1 ? "80" : location.getPort())
				+ location.getPath());
		System.out.println("WSDL location tgextra-crud: "
				+ locationCrud.getProtocol() + "://" + locationCrud.getHost()
				+ ":"
				+ (locationCrud.getPort() == -1 ? "80" : locationCrud.getPort())
				+ locationCrud.getPath());

		initTgextraService(location);

		// Get other needed things.
		rbacSessionId = p.getProperty("sessionId");
		rbacSessionIdInvalid = p.getProperty("sessionIdInvalid");
		junitProject = p.getProperty("junitProject");
		crudSecret = p.getProperty("crudSecret");
		crudSecretInvalid = p.getProperty("crudSecretInvalid");
		logParameter = p.getProperty("logParameter");
		projectIdInvalid = p.getProperty("projectIdInvalid");
		username = p.getProperty("username");
		usernameInvalid = p.getProperty("usernameInvalid");
		usernameSearch = p.getProperty("usernameSearch");

		// addProject();
	}

	/**
	 * @throws Exception
	 */
	@AfterClass
	public static void tearDownAfterClass() throws Exception {
		// deleteObjects();
		// deleteProject();
	}

	/**
	 * <p>
	 * Tests if a SID's user has assigned some roles.
	 * </p>
	 */
	@Test
	public void testTgAssignedRoles() {

		System.out.println("Testing #TGASSIGNEDROLES");

		TgAssignedRolesRequest req = new TgAssignedRolesRequest();
		req.setAuth(rbacSessionId);
		req.setLog(logParameter);

		System.out.println("\tsession ID:    " + req.getAuth());
		System.out.println("\tlog parameter: " + req.getLog());
		System.out.println(TGAuthClientCommon.TABLINE);

		RolesetResponse response = tgextra.tgAssignedRoles(req);

		for (String role : response.getRole()) {
			String data[] = role.split(",");
			if (data.length == 3) {
				System.out.println(
						"\t" + data[0] + "\t" + data[1] + "\t" + data[2]);
			}
		}

		System.out.println(TGAuthClientCommon.TABLINE);
		System.out.println("\tOK");
	}

	/**
	 * <p>
	 * Gets all rights a user has on the project.
	 * </p>
	 * 
	 * @throws info.textgrid.namespaces.middleware.tgauth.AuthenticationFault
	 */
	@Test
	public void testGetRights()
			throws info.textgrid.namespaces.middleware.tgauth.AuthenticationFault {

		System.out.println("Testing #GETRIGHTS");

		GetRightsRequest req = new GetRightsRequest();
		req.setAuth(rbacSessionId);
		req.setLog(logParameter);
		req.setResource(projectId);

		System.out.println("\tsession ID:    " + req.getAuth());
		System.out.println("\tlog parameter: " + req.getLog());
		System.out.println("\tresource:      " + req.getResource());
		System.out.println(TGAuthClientCommon.TABLINE);

		OperationsetResponse response = tgextra.getRights(req);

		for (String operation : response.getOperation()) {
			System.out.println("\t" + operation);
		}

		System.out.println(TGAuthClientCommon.TABLINE);
		System.out.println("\tOK");
	}

	/**
	 * <p>
	 * Gets all roles a user has on the project.
	 * </p>
	 * 
	 * @throws info.textgrid.namespaces.middleware.tgauth.AuthenticationFault
	 * @throws UnknownProjectFault
	 */
	@Test
	public void testGetUserRole()
			throws info.textgrid.namespaces.middleware.tgauth.AuthenticationFault,
			UnknownProjectFault {

		System.out.println("Testing #GETUSERROLE");

		GetUserRoleRequest req = new GetUserRoleRequest();
		req.setAuth(rbacSessionId);
		req.setLog(logParameter);
		req.setProject(projectId);

		System.out.println("\tsession ID:    " + req.getAuth());
		System.out.println("\tlog parameter: " + req.getLog());
		System.out.println("\tproject:       " + req.getProject());
		System.out.println(TGAuthClientCommon.TABLINE);

		GetUserRoleResponse response = tgextra.getUserRole(req);

		for (UserRole role : response.getUserRole()) {
			System.out.print("\tRoles for " + role.getUsername() + ": ");
			for (String r : role.getRoles()) {
				System.out.print(r + " ");
			}
			System.out.println();
		}

		System.out.println(TGAuthClientCommon.TABLINE);
		System.out.println("\tOK");
	}

	/**
	 * <p>
	 * Test the permission the delete a project.
	 * </p>
	 *
	 * @throws info.textgrid.namespaces.middleware.tgauth.UnknownResourceFault
	 * @throws UnknownProjectFault
	 * @throws info.textgrid.namespaces.middleware.tgauth.AuthenticationFault
	 */
	@Test
	public void testTgDeleteProjectPermission()
			throws info.textgrid.namespaces.middleware.tgauth.UnknownResourceFault,
			info.textgrid.namespaces.middleware.tgauth.AuthenticationFault,
			UnknownProjectFault {

		// Get roles first for project.
		GetUserRoleRequest req = new GetUserRoleRequest();
		req.setAuth(rbacSessionId);
		req.setLog(logParameter);
		req.setProject(projectId);

		System.out.println("\tsession ID:    " + req.getAuth());
		System.out.println("\tlog parameter: " + req.getLog());
		System.out.println("\tproject:       " + req.getProject());
		System.out.println(TGAuthClientCommon.TABLINE);

		GetUserRoleResponse response = tgextra.getUserRole(req);

		// Check if the user has the role Project Manager in the role project.
		boolean userHasRoleProjektleiter = false;
		for (UserRole role : response.getUserRole()) {
			if (role.getUsername().equals(username)) {
				for (String r : role.getRoles()) {
					if (r.equals(TGAuthClientCommon.ROLE_MANAGER)) {
						userHasRoleProjektleiter = true;
						System.out.println("\tRole '"
								+ TGAuthClientCommon.ROLE_MANAGER
								+ "' found! Deletion of project allowed!");
					}
				}
			}
		}

		if (!userHasRoleProjektleiter) {
			assertTrue(false);
		}

		System.out.println(TGAuthClientCommon.TABLINE);
		System.out.println("\tOK");
	}

	/**
	 * <p>
	 * Test the tgCheckAccess method with access granted on project#CREATE.
	 * </p>
	 * 
	 * @throws info.textgrid.namespaces.middleware.tgauth.UnknownResourceFault
	 */
	@Test
	public void testTgCheckAccessProjectCreate()
			throws info.textgrid.namespaces.middleware.tgauth.UnknownResourceFault {

		System.out.println(
				"Testing #TGCHECKACCESS with access granted (project#CREATE)");

		TgCheckAccessRequest req = new TgCheckAccessRequest();
		req.setAuth(rbacSessionId);
		req.setLog(logParameter);
		req.setOperation(TGAuthClientCommon.OPERATION_CREATE);
		req.setResource(projectId);

		System.out.println("\tsession ID:    " + req.getAuth());
		System.out.println("\tlog parameter: " + req.getLog());
		System.out.println("\toperation:     " + req.getOperation());
		System.out.println("\tresource:      " + req.getResource());
		System.out.println(TGAuthClientCommon.TABLINE);

		try {
			System.out.println(
					"\tresult: " + tgextra.tgCheckAccess(req).isResult());

			System.out.println(TGAuthClientCommon.TABLINE);
			System.out.println("\tOK");

		} catch (info.textgrid.namespaces.middleware.tgauth.UnknownResourceFault e) {
			System.err.println("\tARGL! ERROR! " + e.getClass().getName() + ": "
					+ e.getMessage());
			throw e;
		}
	}

	/**
	 * <p>
	 * Test the tgCheckAccess method with invalid SID.
	 * </p>
	 * 
	 * @throws info.textgrid.namespaces.middleware.tgauth.UnknownResourceFault
	 */
	@Test
	public void testTgCheckAccessDenied()
			throws info.textgrid.namespaces.middleware.tgauth.UnknownResourceFault {

		System.out.println(
				"Testing #TGCHECKACCESS with access denied (project#CREATE)");

		TgCheckAccessRequest req = new TgCheckAccessRequest();
		req.setAuth(rbacSessionIdInvalid);
		req.setLog(logParameter);
		req.setOperation(TGAuthClientCommon.OPERATION_CREATE);
		req.setResource(projectId);

		System.out.println("\tsession ID:    " + req.getAuth());
		System.out.println("\tlog parameter: " + req.getLog());
		System.out.println("\toperation:     " + req.getOperation());
		System.out.println("\tresource:      " + req.getResource());
		System.out.println(TGAuthClientCommon.TABLINE);

		try {
			boolean result = tgextra.tgCheckAccess(req).isResult();
			System.out.println("\tresult: " + result);

			if (result) {
				assertTrue(false);
			}

			System.out.println(TGAuthClientCommon.TABLINE);
			System.out.println("\tOK");

		} catch (info.textgrid.namespaces.middleware.tgauth.UnknownResourceFault e) {
			System.err.println("\tARGL! ERROR! " + e.getClass().getName() + ": "
					+ e.getMessage());
			throw e;
		}
	}

	/**
	 * <p>
	 * Test the tgCheckAccess method with invalid SID.
	 * </p>
	 * 
	 * TODO
	 * 
	 * @throws info.textgrid.namespaces.middleware.tgauth.UnknownResourceFault
	 */
	@Test
	@Ignore
	public void testTgCheckAccessFile()
			throws info.textgrid.namespaces.middleware.tgauth.UnknownResourceFault {

		System.out.println(
				"Testing #TGCHECKACCESS with access granted (object#DELETE)");

		TgCheckAccessRequest req = new TgCheckAccessRequest();
		req.setAuth(rbacSessionIdInvalid);
		req.setLog(logParameter);
		req.setOperation(TGAuthClientCommon.OPERATION_DELETE);
		// FIXME Create object first :-)
		req.setResource("textgrid:07.0");

		System.out.println("\tsession ID:    " + req.getAuth());
		System.out.println("\tlog parameter: " + req.getLog());
		System.out.println("\toperation:     " + req.getOperation());
		System.out.println("\tresource:      " + req.getResource());
		System.out.println(TGAuthClientCommon.TABLINE);

		try {
			boolean result = tgextra.tgCheckAccess(req).isResult();
			System.out.println("\tresult: " + result);

			if (result) {
				assertTrue(false);
			}

			System.out.println(TGAuthClientCommon.TABLINE);
			System.out.println("\tOK");

		} catch (info.textgrid.namespaces.middleware.tgauth.UnknownResourceFault e) {
			System.err.println("\tARGL! ERROR! " + e.getClass().getName() + ": "
					+ e.getMessage());
			throw e;
		}
	}

	/**
	 * <p>
	 * Test the tgCheckAccess method with invalid resource.
	 * </p>
	 * 
	 * @throws info.textgrid.namespaces.middleware.tgauth.UnknownResourceFault
	 */
	@Test(expected = info.textgrid.namespaces.middleware.tgauth.UnknownResourceFault.class)
	public void testTgCheckAccessUnknownResource()
			throws info.textgrid.namespaces.middleware.tgauth.UnknownResourceFault {

		System.out.println(
				"Testing #TGCHECKACCESS with unknown resource (project#CREATE)");

		TgCheckAccessRequest req = new TgCheckAccessRequest();
		req.setAuth(rbacSessionId);
		req.setLog(logParameter);
		req.setOperation(TGAuthClientCommon.OPERATION_CREATE);
		req.setResource(projectIdInvalid);

		System.out.println("\tsession ID:    " + req.getAuth());
		System.out.println("\tlog parameter: " + req.getLog());
		System.out.println("\toperation:     " + req.getOperation());
		System.out.println("\tresource:      " + req.getResource());
		System.out.println(TGAuthClientCommon.TABLINE);

		try {
			tgextra.tgCheckAccess(req).isResult();
		} catch (info.textgrid.namespaces.middleware.tgauth.UnknownResourceFault e) {
			System.out.println("\tOK");
			throw e;
		}
	}

	/**
	 * <p>
	 * Test the userExists method with valid user.
	 * </p>
	 * 
	 * @throws info.textgrid.namespaces.middleware.tgauth.AuthenticationFault
	 */
	@Test
	public void testUserExists()
			throws info.textgrid.namespaces.middleware.tgauth.AuthenticationFault {

		System.out.println("Testing #USEREXISTS with existing username");

		UserExistsRequest req = new UserExistsRequest();
		req.setAuth(rbacSessionId);
		req.setLog(logParameter);
		req.setUsername(username);

		System.out.println("\tsession ID:    " + req.getAuth());
		System.out.println("\tlog parameter: " + req.getLog());
		System.out.println("\tusername:      " + req.getUsername());
		System.out.println(TGAuthClientCommon.TABLINE);

		boolean result = tgextra.userExists(req).isResult();
		System.out.println("\tresult: " + result);

		if (!result) {
			assertTrue(false);
		}

		System.out.println(TGAuthClientCommon.TABLINE);
		System.out.println("\tOK");
	}

	/**
	 * <p>
	 * Test the userExists method with valid user.
	 * </p>
	 * 
	 * @throws info.textgrid.namespaces.middleware.tgauth.AuthenticationFault
	 */
	@Test
	public void testUserExistsInvalidUsername()
			throws info.textgrid.namespaces.middleware.tgauth.AuthenticationFault {

		System.out.println("Testing #USEREXISTS with invalid username");

		UserExistsRequest req = new UserExistsRequest();
		req.setAuth(rbacSessionId);
		req.setLog(logParameter);
		req.setUsername(usernameInvalid);

		System.out.println("\tsession ID:    " + req.getAuth());
		System.out.println("\tlog parameter: " + req.getLog());
		System.out.println("\tusername:      " + req.getUsername());
		System.out.println(TGAuthClientCommon.TABLINE);

		boolean result = tgextra.userExists(req).isResult();
		System.out.println("\tresult: " + result);

		if (result) {
			assertTrue(false);
		}

		System.out.println(TGAuthClientCommon.TABLINE);
		System.out.println("\tOK");
	}

	/**
	 * <p>
	 * Test searching for names and email addresses.
	 * </p>
	 * 
	 * @throws AuthenticationFault
	 */
	@Test
	public void testGetIDs() {

		System.out.println("Testing #GETID name");

		// Create getEPPN request.
		GetIDsRequest req = new GetIDsRequest();
		req.setAuth(rbacSessionId);
		req.setLog(logParameter);
		req.setName(username);

		System.out.println("\tsession ID:    " + req.getAuth());
		System.out.println("\tlog parameter: " + req.getLog());
		System.out.println("\tusername:      " + req.getName());
		System.out.println(TGAuthClientCommon.TABLINE);

		GetIDsResponse res = tgextra.getIDs(req);

		// Get first (and only) entry for given ePPn.
		UserDetail det = res.getUserdetails().get(0);
		System.out.println(
				"\tuser details:  " + det.getEPPN() + " " + det.getMail() + " "
						+ det.getName() + " " + det.getOrganisation() + " "
						+ det.isAgreesearch() + " " + det.isUsersupplieddata());

		System.out.println(TGAuthClientCommon.TABLINE);

		if (!det.getEPPN().equals(username)) {
			System.out.println("\tERROR!");
			assertTrue(false);
		} else {
			System.out.println("\tOK");
		}
	}

	/**
	 * <p>
	 * Test searching for agreeSearch attribute.
	 * </p>
	 * 
	 * @throws AuthenticationFault
	 */
	@Test
	public void testGetIDsAgreeSearch() {

		System.out.println("Testing #GETID agreeSearch");

		// Create getEPPN request.
		GetIDsRequest req = new GetIDsRequest();
		req.setAuth(rbacSessionId);
		req.setLog(logParameter);
		req.setName(username);

		System.out.println("\tsession ID:    " + req.getAuth());
		System.out.println("\tlog parameter: " + req.getLog());
		System.out.println("\tusername:      " + req.getName());
		System.out.println(TGAuthClientCommon.TABLINE);

		GetIDsResponse res = tgextra.getIDs(req);

		// Get first (and only) entry for given ePPn.
		UserDetail det = res.getUserdetails().get(0);
		System.out.println(
				"\tuser details:  " + det.getEPPN() + " " + det.getMail() + " "
						+ det.getName() + " " + det.getOrganisation() + " "
						+ det.isAgreesearch() + " " + det.isUsersupplieddata());

		System.out.println(TGAuthClientCommon.TABLINE);

		if (det.isAgreesearch() == null) {
			System.out.println("\tERROR! Attribtue agreeSearch is not set!");
			assertTrue(false);
		} else {
			System.out.println("\tOK");
		}
	}

	/**
	 * <p>
	 * Test searching for part of username, truncated!
	 * </p>
	 * 
	 * @throws AuthenticationFault
	 */
	@Test
	public void testGetIDsSearchUserTruncated() {

		System.out.println("Testing #GETID search user truncated");

		// Create getEPPN request.
		GetIDsRequest req = new GetIDsRequest();
		req.setAuth(rbacSessionId);
		req.setLog(logParameter);
		String trunk = "*" + usernameSearch + "*";
		req.setName(trunk);

		System.out.println("\tsession ID:    " + req.getAuth());
		System.out.println("\tlog parameter: " + req.getLog());
		System.out.println("\tusername:      " + req.getName());
		System.out.println(TGAuthClientCommon.TABLINE);

		GetIDsResponse res = tgextra.getIDs(req);

		// Get first (and only) entry for given ePPn.
		if (res.getUserdetails().size() != 0) {
			System.out.println("\tuser details:");
			for (UserDetail det : res.getUserdetails()) {
				System.out.println("\t\t" + det.getEPPN() + " " + det.getMail()
						+ " " + det.getName() + " " + det.getOrganisation()
						+ " " + det.isAgreesearch() + " "
						+ det.isUsersupplieddata());
			}
		} else {
			System.out.println("\tnothing found");
		}

		System.out.println(TGAuthClientCommon.TABLINE);

		if (res.getUserdetails().size() == 0) {
			System.out.println(
					"\tERROR! No users found containing " + trunk + "!");
			assertTrue(false);
		} else {
			System.out.println("\tOK");
		}
	}

	/**
	 * <p>
	 * Test searching for part of username, not truncated!
	 * </p>
	 * 
	 * @throws AuthenticationFault
	 */
	@Test
	public void testGetIDsSearchUserNoTruncated() {

		System.out.println("Testing #GETID search user not truncated");

		// Create getEPPN request.
		GetIDsRequest req = new GetIDsRequest();
		req.setAuth(rbacSessionId);
		req.setLog(logParameter);
		req.setName(usernameSearch);

		System.out.println("\tsession ID:    " + req.getAuth());
		System.out.println("\tlog parameter: " + req.getLog());
		System.out.println("\tusername:      " + req.getName());
		System.out.println(TGAuthClientCommon.TABLINE);

		GetIDsResponse res = tgextra.getIDs(req);

		// Get first (and only) entry for given ePPn.
		if (res.getUserdetails().size() != 0) {
			System.out.println("\tuser details:");
			for (UserDetail det : res.getUserdetails()) {
				System.out.println("\t\t" + det.getEPPN() + " " + det.getMail()
						+ " " + det.getName() + " " + det.getOrganisation()
						+ " " + det.isAgreesearch() + " "
						+ det.isUsersupplieddata());
			}
		} else {
			System.out.println("nothing found");
		}

		System.out.println(TGAuthClientCommon.TABLINE);

		if (res.getUserdetails().size() == 0) {
			System.out.println("\tERROR! No users found containing "
					+ usernameSearch + "!");
			assertTrue(false);
		} else {
			System.out.println("\tOK");
		}
	}

	/**
	 * <p>
	 * Test getting all the objects from one project.
	 * </p>
	 * 
	 */
	@Test
	public void getAllObjects() {

		System.out.println("Getting all objects from project: " + projectId);

		GetObjectsRequest req = new GetObjectsRequest();
		req.setAuth(rbacSessionId);
		req.setLog(logParameter);
		req.setProject(projectId);

		try {
			ResourcesetResponse res = tgextra.getObjects(req);
			int size = res.getResource().size();

			System.out.println("\tProject has got " + size + " entr"
					+ (size != 1 ? "ies" : "y"));
			System.out.println("\tOK");

		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();

			assertTrue(false);
		}
	}

	/**
	 * <p>
	 * ONLINE TEST: Test getting all the objects from large project.
	 * </p>
	 * 
	 */
	@Test
	// @Ignore
	public void getAllObjectsDigibib() {

		String digibib = "TGPR-372fe6dc-57f2-6cd4-01b5-2c4bbefcfd3c";
		String diginame = "Digitale Bibliothek";

		// String digibib = "TGPR-53fd0966-4bae-45e4-519f-57fcf48e3249";
		// String diginame = "fus tolles projekt";

		System.out.println("\tGetting all objects from project '" + diginame
				+ "': " + digibib);

		GetObjectsRequest req = new GetObjectsRequest();
		req.setAuth(rbacSessionId);
		req.setLog(logParameter);
		req.setProject(digibib);

		try {
			ResourcesetResponse res = tgextra.getObjects(req);
			int size = res.getResource().size();

			System.out.println("Project has got " + size + " entr"
					+ (size != 1 ? "ies" : "y"));
			System.out.println("\tOK");

		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();

			assertTrue(false);
		}
	}

	// **
	// PRIVATE METHODS
	// **

	/**
	 * <p>
	 * Adds a project for testing.
	 * </p>
	 */
	private static void addProject() {

		GetAllProjectsRequest allProjetsRequest = new GetAllProjectsRequest();
		allProjetsRequest.setAuth(rbacSessionId);
		allProjetsRequest.setLog(logParameter);
		List<ProjectInfo> allProjects = tgextra
				.getAllProjects(allProjetsRequest).getProject();

		System.out.println(
				"Checking for Junit test project '" + junitProject + "'");

		boolean projectExisting = false;
		for (ProjectInfo pi : allProjects) {
			if (pi.getName().equals(junitProject)) {
				projectExisting = true;
				projectId = pi.getId();
			}
		}
		if (projectExisting) {
			System.out.println("\tProject " + junitProject + " already exists: "
					+ projectId);
		} else {
			CreateProjectRequest junitProject1Request = new CreateProjectRequest();
			junitProject1Request.setAuth(rbacSessionId);
			junitProject1Request.setLog(logParameter);
			junitProject1Request.setName(junitProject);
			junitProject1Request.setDescription(junitProject);
			CreateProjectResponse project1Response = tgextra
					.createProject(junitProject1Request);
			projectId = project1Response.getProjectId();

			System.out.println(
					"\tProject " + junitProject + " created: " + projectId);

			try {
				// Add user1 to project1 as Editor, Administrator, and Project
				// Manager.
				AddMemberRequest addMemberRequest = new AddMemberRequest();
				String roleEditor = TGAuthClientCommon.ROLE_EDITOR + ","
						+ projectId + ",Projekt-Teilnehmer";
				addMemberRequest.setAuth(rbacSessionId);
				addMemberRequest.setLog(logParameter);
				addMemberRequest.setRole(roleEditor);
				addMemberRequest.setUsername(username);
				tgextra.addMember(addMemberRequest);

				String roleAdmin = TGAuthClientCommon.ROLE_ADMIN + ","
						+ projectId + ",Projekt-Teilnehmer";
				addMemberRequest.setRole(roleAdmin);
				tgextra.addMember(addMemberRequest);

				String roleManager = TGAuthClientCommon.ROLE_MANAGER + ","
						+ projectId + ",Projekt-Teilnehmer";
				addMemberRequest.setRole(roleManager);
				tgextra.addMember(addMemberRequest);

				// Observer
				String roleObserver = TGAuthClientCommon.ROLE_OBSERVER + ","
						+ projectId + ",Projekt-Teilnehmer";
				addMemberRequest.setRole(roleObserver);
				tgextra.addMember(addMemberRequest);

				System.out.println("\tAdded " + username + " to project "
						+ junitProject + " with roles '"
						+ TGAuthClientCommon.ROLE_EDITOR + "', '"
						+ TGAuthClientCommon.ROLE_ADMIN + "', '"
						+ TGAuthClientCommon.ROLE_MANAGER + "', and '"
						+ TGAuthClientCommon.ROLE_OBSERVER + "'");
			} catch (RbacFault e) {
				System.out.println("\tRBAC Fault: " + e.getMessage());
			}
		}
	}

	/**
	 * <p>
	 * Test deleting the test project.
	 * </p>
	 * 
	 * @throws info.textgrid.namespaces.middleware.tgauth.AuthenticationFault
	 * @throws NotEmptyFault
	 */
	private static void deleteProject()
			throws info.textgrid.namespaces.middleware.tgauth.AuthenticationFault,
			NotEmptyFault {

		System.out.println(
				"Deleting test project " + junitProject + ": " + projectId);

		DeleteProjectRequest req = new DeleteProjectRequest();
		req.setAuth(rbacSessionId);
		req.setLog(logParameter);
		req.setProject(projectId);

		tgextra.deleteProject(req);

		System.out.println("\tOK");
	}

	/**
	 * <p>
	 * Initialises a regular TG-extra stub.
	 * </p>
	 * 
	 * @param theEndpoint
	 */
	private static void initTgextraService(URL theEndpoint) {

		if (tgextra == null) {

			// Create TG-extra service stubs.
			tgextra = TGAuthClientUtilities.getTgextra(theEndpoint);

			System.out.println("TG-auth stub created");
		} else {
			System.out.println("Using existing TG-auth stub");
		}
	}

}
